export default function cuepointsReader() {
    const list = [];
    let reader = null;
    let dispatch;
    let currentPosition = 0;
    let lastPosition = -1;
    let tolerance = 0.2;

    function add(position, name, data) {
        list.push({position, name, data});

        list.sort((a, b) => a.position - b.position);

        return reader;
    }

    function onCuepoint(fn, thisArg) {
        if (fn) {
            dispatch = thisArg ? fn.bind(thisArg) : fn;
        } else {
            dispatch = null;
        }
        return reader;
    }

    function reset() {
        currentPosition = 0;
        lastPosition = -1;
        return reader;
    }

    function removeAll() {
        list.length = 0;
        return reset();
    }

    function setTolerance(value) {
        tolerance = value;
        return reader;
    }

    function inRange(cuepointPos, currentPos, lastPos) {
        if (cuepointPos > currentPos) {
            return false;
        }
        if (cuepointPos <= lastPos) {
            return false;
        }

        let diff = cuepointPos - currentPos;
        if (diff < 0) {
            diff = -diff;
        }
        return diff <= tolerance;
    }

    function check(currentPos, lastPos) {
        if (currentPos <= lastPos) {
            return;
        }
        if (typeof dispatch !== 'function') {
            return;
        }

        list.some(item => {
            if (inRange(item.position, currentPos, lastPos)) {
                dispatch(item);
                return true;
            }
            return false;
        });
    }

    function update(position) {
        currentPosition = position;
        check(currentPosition, lastPosition);
        lastPosition = currentPosition;
        return reader;
    }

    reader = Object.freeze({
        add,
        onCuepoint,
        removeAll,
        reset,
        setTolerance,
        update
    });

    return reader;
}
